/* Приложение страницы с настройками освещения */

const App = {
    delimiters: ['{|', '|}'],
    el: '#incubator-app',
    provide: function () {
        return {
            'copyJson': this.copyJson,
            'addOrEditCycle': this.addOrEditCycle,
            'getNextTriggerIndex': this.getNextTriggerIndex,
        }
    },
    data: {
        cycleList: [],
        cycle: {triggers: {}}, // текущий выбранный цикл, ссылка на объект из cycleList
        cycleCopy: {}, // независимая копия cycle, которую можно менять, а в cycle сохранять по кнопке
        locationList: [],
        location: {}, // текущая выбранная локация
        currentDay: {value: 0, displayValue: 1}, // текущий выбранный день,
        scheduledLocations: [],
        addModal: undefined,
    },
    methods: {
        httpRequest(url, method='POST', body={}, async=false) {
            // POST-запрос на указанный url. Параметры необходимо передавать также в url
            let resp = {result: null, error: null};
            if (method == 'POST') {
                let request = new XMLHttpRequest();
                request.open(method, url, async);
                request.withCredentials = true;
                request.setRequestHeader('Content-Type', 'application/json; charset=UTF-8');
                request.setRequestHeader('X-CSRFToken', this.getCookie('csrftoken'));
                request.send(JSON.stringify(body));
                if (request.status == 200) {
                    resp.result = JSON.parse(request.responseText);
                } else {
                    resp.error = request.responseText;
                    console.error(resp.error);
                }
                return resp;
            }
        },
        copyJson(json) {
            return JSON.parse(
                JSON.stringify(json)
            )
        },
        getData() {
            const url = '/spread/incubator/config/';
            let resp = this.httpRequest(url);
            if (resp.error) {
                return;
            }
            this.locationList = resp.result.locations.filter(it => it.cycle_id==null);
            this.scheduledLocations = resp.result.locations.filter(it => it.cycle_id!=null);
            this.cycleList = resp.result.cycles;
            return;
        },
        addLocationCycle() {
            this.location.cycle_id = this.cycle.id;
            this.location.start_time = this.location.start_time || moment().format('YYYY-MM-DD')
            this.updateLocation(this.location)
            this.location = {};
            this.cycle = {};
            // запрашиваем обновленные данные
            this.getData();
        },
        deleteLocationCycle(location) {
            location.cycle_id = null;
            this.updateLocation(location);
            this.getData();
        },
        setActive(location) {
            location.active = !location.active;
            this.updateLocation(location);
            this.getData();
        },
       setRecursive(location) {
            location.recursive = !location.recursive;
            this.updateLocation(location);
            this.getData();
        },
        updateLocation(location) {
            const url = '/spread/incubator/update/location/';
            let resp = this.httpRequest(url, 'POST', location);
            return resp.result;
        },
        updateCycle(cycle=undefined) {
            cycle = cycle || this.cycle;
            const url = '/spread/incubator/update/cycle/';
            let resp = this.httpRequest(url, 'POST', cycle);
            return resp.result;
        },
        addOrEditCycle(cycle=undefined) {
            this.updateCycle(cycle);
            this.addModal.hide();
            // this.closeEditCycleModal();
            this.getData();
            cycle = {triggers:{}};
        },
        copyCycle(cycle) {
            cycle.id = '';
            cycle.name += ' (копия)';
            this.updateCycle(cycle);
            this.getData();
        },
        deleteCycle(cycle) {
            let body = {id: cycle.id};
            const url = '/spread/incubator/delete/cycle/';
            let resp = this.httpRequest(url, 'POST', body);
            this.getData();
            return resp.result;
        },
        currentCycleDay(location)  {
            // находим разницу между сегодняшним днем и началом цикла
            // возвращаем остаток от деления разницы на продолжительность цикла - текущий день цикла
            const start = location.start_time ? moment(location.start_time) : moment();
            const today = moment();
            const diff = moment.duration(today - start).days();
            const period = location.cycle__period;
            return diff % period + 1; // получаем номер дня с 0, а выводим с 1
        },
        cycleDays(location) {
            const period = location.cycle__period;
            let days = Array.from(
                {length: period},
                (_, index) => ({value: index, displayValue: index+1})
            )
            return days;
        },
        setCurrentCycleDay(location, day) {
            // вычтем из "сегодня" номер дня и получим новую дату начала цикла
            // запишем ее для данной локации
            const newStart = moment().subtract(day.value, 'days')
            location.start_time = newStart.format('YYYY-MM-DD');
            this.updateLocation(location);
            this.getData();
        },
        getNextTriggerIndex(triggers) {
             let index = Object.keys(triggers).length > 0 ?
                Object.keys(triggers).reduce((a, b) => parseInt(a) > parseInt(b) ? a : b) : 0;
             return parseInt(index) + 1;
        },
        getCookie(key) {
            // костыль для извлечения куки по ключу
            const value = `; ${document.cookie}`;
            const parts = value.split(`; ${key}=`);
            if (parts.length === 2) return parts.pop().split(';').shift();
        }
    },
    created() {
        this.getData();
    },
    mounted() {
        this.addModal = new bootstrap.Modal('#add-cycle', {});
    },
}
