import logging

import spread_bolid as bolid
from spread_core.bam import const
from spread_core.bam.bolid import providers
from spread_core.bam.bolid.interface import ModbusTcp
from ..managers import Manager


class Bolid(Manager):

    def __init__(self, p_id, m_id):
        super().__init__(p_id, m_id)
        self.event_desc_length = None
        self.slave_id = None

    def parse(self, data):
        super(Bolid, self).parse(data)
        self.slave_id = data[const.ATTRIBUTES][const.SLAVE_ID]

    def set_interface(self):
        self.send_interface = ModbusTcp()

    def pre_info(self):
        super(Bolid, self).pre_info()
        if self._stopped:
            return

        self.make_request(bolid.commands.EventDescLength())
        self.make_request(bolid.commands.MaxRelayCount())

    def survey(self):
        i = self.send_interface
        event_count = i.send(bolid.commands.NewEventCount(slave_id=self.slave_id))
        for _ in range(event_count):
            try:
                event = i.send(bolid.commands.FirstEvent(self.event_desc_length, slave_id=self.slave_id))
                if event.code != 254:
                    self.on_event(event)
                i.send(bolid.commands.MarkEvent(event.index, slave_id=self.slave_id))
            except Exception as ex:
                logging.exception(ex)

        yield

    def make_request(self, cmd: bolid.commands.Command, *args, **kwargs):
        cmd.set_slave_id(self.slave_id)
        response = self.send_interface.send(cmd, *args, **kwargs)
        self.on_request_answer(cmd, response)
        return response

    def on_event(self, event: bolid.events.Event):
        if event.area is not None:
            for provider in self.providers_by_address(event.area):
                if isinstance(provider, providers.BolidArea):
                    provider.on_event(event)
        elif event.relay_index is not None:
            for provider in self.providers_by_address(event.relay_index):
                if isinstance(provider, providers.BolidRelay):
                    provider.on_event(event)

    def on_request_answer(self, request: bolid.commands.Command, response, sig=None):
        if isinstance(request, bolid.commands.EventDescLength):
            self.event_desc_length = response

        elif isinstance(request, bolid.commands.MaxRelayCount):
            self.make_request(bolid.commands.ReadGroupRelay(1, response))

        elif isinstance(request, bolid.commands.ReadGroupRelay):
            for index in range(request.count):
                for provider in self.providers_by_address(request.start_index + index):
                    if isinstance(provider, providers.BolidRelay):
                        provider.on_request_answer(request, [response[index]], sig=sig)
