import logging

import spread_dali as dali
from spread_core.bam.dali.store_interface import DaliWarehouseInterface
from spread_core.tools.settings import config, LOCAL_ADAPTER, LEVEL_SYNC_DELAY, SURVEY_RATIO
from .providers import sensors
from ..managers import Manager

DALI1 = 1
DALI2 = 2


class Dali(Manager, DaliWarehouseInterface):
    DTR0 = 0
    DTR1 = 1
    DTR2 = 2

    level_sync_delay = 1

    def __init__(self, p_id, m_id):
        self.survey_ratio = 1
        self.status_iteration = 0

        super().__init__(p_id, m_id)
        DaliWarehouseInterface.__init__(self)
        self.dtr_values = [None, [None]*3, [None]*3]

    def set_dtr_value(self, dali_type, index, value):
        self.dtr_values[dali_type][index] = value

    def get_dtr_value(self, dali_type, index):
        return self.dtr_values[dali_type][index]

    def set_interface(self):
        if config.get(LOCAL_ADAPTER, False):
            i = dali.interface.DaliCanAdapter()
        else:
            i = dali.interface.BrokerAdapter(self.mqttc)

        self.send_interface = i
        logging.info('Bus interface is "%s"' % self._send_interface.__class__.__name__)

    def update_config(self):
        super(Dali, self).update_config()
        if self._send_interface and isinstance(self._send_interface, dali.interface.DaliCanAdapter) != config.get(LOCAL_ADAPTER, False):
            self.set_interface()

        self.level_sync_delay = config.get(LEVEL_SYNC_DELAY, self.level_sync_delay)
        self.survey_ratio = config.get(SURVEY_RATIO, self.survey_ratio)
        from spread_core.bam.dali.tuner import TunerLimited
        TunerLimited.update_config()

    def on_external_command(self, command, response, *args):
        from spread_core.bam.dali.external_dispatcher import ExternalDispatcher

        dispatcher = ExternalDispatcher(self)
        dispatcher.dispatch(command, response, *args)

    def on_event(self, event: dali.device.events.Event):
        if isinstance(event.source1, dali.address.Short):
            for _provider in self.providers_by_address(DALI2, event.source1):
                if isinstance(_provider, sensors.DaliSensor) and _provider.compare_instance(event.source2):
                    event.owner_type = _provider.type_id
                    logging.debug(f'<- {event}')
                    _provider.on_event(event)
                    return
        elif isinstance(event.source1, dali.address.Group):
            for _provider in self.get_group_members(DALI2, event.source1.group):
                if isinstance(_provider, sensors.DaliSensor) and _provider.compare_instance(event.source2):
                    event.owner_type = _provider.type_id
                    _provider.on_event(event)

        logging.debug(f'<- {event}')

    def status_survey(self):
        for provider in self.get_all_devices_of_type(DALI1):
            yield from provider.on_status_survey()

        self.status_iteration += 1

        if self.status_iteration == self.survey_ratio:
            self.status_iteration = 0
            for provider in self.get_all_devices_of_type(DALI2):
                yield from provider.on_status_survey()
