from spread_core.mqtt.spread.topic import State
from spread_core.mqtt.spread.variable import Variable
from .enginery import Enginery, _EngineryFunit
from ..recipe import Recipe


class _AlarmSensor(Enginery):
    SOLID = 'Solid'
    GUARD_TYPE = 'isGuarded'
    TRIGGER_TYPE = 'isTriggered'

    def __init__(self, _id: int, **kwargs):
        super(_AlarmSensor, self).__init__(_id, **kwargs)
        self._trigger = None

    def set_recipe(self, ingredient_type, recipe: Recipe):
        if ingredient_type == self.SOLID:
            self._recipe = recipe

        elif ingredient_type == self.GUARD_TYPE:
            self._recipe = recipe

        elif ingredient_type == self.TRIGGER_TYPE:
            self._trigger = recipe

        else:
            raise ValueError('unknown ingredient type: "%s"' % ingredient_type)

    def recipe_for_funit(self, funit):
        if funit == Guard and self.recipe:
            if self.recipe.provider_type in ('DaliRelay', 'KnxRelay'):
                return 'On', self.recipe
            else:
                return funit, self.recipe
        else:
            return funit, None

    def on_state(self, topic: State, variable: Variable):
        if topic.entity_address.funit_type == 'On':
            if self.recipe and topic.entity_address.id == self.recipe.id:
                self.publish_state(Guard, None if variable.invalid else variable.value)

            elif self._trigger and topic.entity_address.id == self._trigger.id:
                self.publish_state(Triggered, None if variable.invalid else variable.value)

        else:
            super(_AlarmSensor, self).on_state(topic, variable)

        for subginery in self._subgineries:
            subginery.on_level_changed(self, topic.entity_address.funit_type)


class IntruderSensor(_AlarmSensor):
    pass


class FireSensor(_AlarmSensor):
    pass


class LeakageSensor(_AlarmSensor):
    pass


class _AlarmSensorFunit(_EngineryFunit):
    pass


_AlarmSensor.funit_class = _AlarmSensorFunit


class Guard(_AlarmSensorFunit):
    value_type = bool


class Triggered(_AlarmSensorFunit):
    value_type = bool

    def set(self):
        raise self.NotImplemented()


_AlarmSensor.settable_units = [Guard]

