from .subginery import Subginery, _SubgineriesFunit
from ..engineries import climate
from spread_core.mqtt.spread.topic import State
from spread_core.mqtt.spread.variable import Variable


class Climate(Subginery):
    def on_level_changed(self, eng, eng_funit_type=None):
        value = None
        state_changed = False
        invalid = False
        inconsistent = False

        if isinstance(eng, climate.TemperatureSensor):
            if eng_funit_type == climate.Temperature:
                funit = str(Temperature)
            else:
                return

            total = 0
            n = 0
            for current_eng in self.engineries:
                if isinstance(current_eng, climate.TemperatureSensor):
                    _value = current_eng.values.get(funit)
                    if _value:
                        if value is None:
                            value = _value
                        elif value != _value:
                            inconsistent = True

                        total += _value
                        n += 1
                    else:
                        invalid = True
            if n == 0:
                value = None
            else:
                value = int(total / n)

        elif isinstance(eng, climate._Climate):
            if isinstance(eng, climate.Fan):
                if eng_funit_type == climate.On:
                    funit = FansOn
                else:
                    return

            elif isinstance(eng, climate.HeatedFloor):
                if eng_funit_type == climate.On:
                    funit = HeatedFloorsOn
                else:
                    return

            elif isinstance(eng, climate.Thermoregulator):
                if eng_funit_type == climate.On:
                    funit = ThermoregulatorsOn

                elif eng_funit_type == climate.TargetTemperature:
                    funit = TargetTemperature

                elif eng_funit_type == climate.Preset:
                    funit = Preset

                elif eng_funit_type == climate.Mode:
                    funit = Mode

                elif eng_funit_type == climate.FanSpeedMode:
                    funit = FanSpeedMode

                elif eng_funit_type == climate.LouverMode:
                    funit = LouverMode

                else:
                    return
            else:
                return

            funit = str(funit)
            for current_eng in self.engineries:
                if isinstance(current_eng, eng.__class__):
                    _value = current_eng.values.get(eng_funit_type)
                    if _value is None:
                        invalid = True
                    
                    elif value is None:
                        value = _value

                    elif value != _value:
                        inconsistent = True
                    
                    if invalid and inconsistent:
                        break

            if inconsistent:
                value = None

        else:
            return

        if invalid:
            if funit not in self._invalid:
                self._invalid.append(funit)
                state_changed = True
        else:
            if funit in self._invalid:
                self._invalid.remove(funit)
                state_changed = True

        if inconsistent:
            if funit not in self._inconsistent:
                self._inconsistent.append(funit)
                state_changed = True
        else:
            if funit in self._inconsistent:
                self._inconsistent.remove(funit)
                state_changed = True

        if self.get_value(funit) != value:
            self.set_value(funit, value)
            state_changed = True

        if state_changed:
            self.adapter.publish(State(self.get_address(funit)), Variable(value, invalid=invalid, inconsistent=inconsistent, subginery=True), retain=True, delay=0)


class _ClimatingFunit(_SubgineriesFunit):
    entity_type = Climate
    target_class = climate._Climate


Climate.funit_class = _ClimatingFunit


class Temperature(_ClimatingFunit):
    target_class = climate.TemperatureSensor
    target_command = climate.Temperature

    def set(self):
        raise self.NotImplemented()


class TargetTemperature(_ClimatingFunit):
    target_class = climate.Thermoregulator
    target_command = climate.TargetTemperature


class ThermoregulatorsOn(_ClimatingFunit):
    target_class = climate.Thermoregulator
    target_command = climate.On


class Preset(_ClimatingFunit):
    target_class = climate.Thermoregulator
    target_command = climate.Preset


class Mode(_ClimatingFunit):
    target_class = climate.Thermoregulator
    target_command = climate.Mode


class FanSpeedMode(_ClimatingFunit):
    target_class = climate.Thermoregulator
    target_command = climate.FanSpeedMode


class LouverMode(_ClimatingFunit):
    target_class = climate.Thermoregulator
    target_command = climate.LouverMode


class FansOn(_ClimatingFunit):
    target_class = climate.Fan
    target_command = climate.On


class HeatedFloorsOn(_ClimatingFunit):
    target_class = climate.HeatedFloor
    target_command = climate.On


Climate.settable_units = [
    TargetTemperature,
    ThermoregulatorsOn,
    Preset,
    Mode,
    FanSpeedMode,
    LouverMode,
    FansOn,
    HeatedFloorsOn
]
