import logging

from spread_core.errors.project_errors import CommandError, EntityNotPresent
from spread_core.mqtt.spread import address
from spread_core.mqtt.spread.topic import State
from spread_core.mqtt.spread.variable import Variable
from ..entities import Entity, Funit


class Subginery(Entity):
    def __init__(self, _id: int, location_id: int, enginery_ids: [int], **kwargs):
        super().__init__(_id, **kwargs)
        self._location_id = location_id
        self._enginery_ids = enginery_ids
        self.adapter = None

    @staticmethod
    def parse_args(locationID: int, engineries: [int], **kwargs) -> dict:
        return dict(
            location_id=int(locationID),
            enginery_ids=list(engineries)
        )

    @classmethod
    def get_address_broadcast(cls, p_id, funit_type='+'):
        return address.SubgineryAddress.broadcast(p_id, '+', cls.__name__, funit_type)

    def set_adapter(self, adapter):
        self.adapter = adapter

        for enginery_id in self._enginery_ids:
            try:
                self.adapter.get_entity(enginery_id).set_subginery(self)
            except EntityNotPresent:
                logging.warning(f'Enginery({enginery_id}) not present, but include in {self}')

    def publish_state(self, funit_type, value, delay=0):
        if self.get_value(funit_type) != value:
            self.set_value(funit_type, value)
            self.adapter.publish(State(self.get_address(funit_type)), Variable(value), retain=True, delay=delay)

    def get_address(self, funit_type):
        return address.SubgineryAddress(self.project_id, self._location_id, self.__class__.__name__, funit_type)

    def get_enginery(self, enginery_id: int):
        return self.adapter.get_entity(enginery_id)

    @property
    def project_id(self):
        return self.adapter.id

    @property
    def location_id(self):
        return self._location_id

    @property
    def engineries(self):
        """ Список инженерных объектов, включенных в эту подсистему """

        for enginery_id in self._enginery_ids:
            try:
                yield self.adapter.get_entity(enginery_id)
            except EntityNotPresent:
                pass

    def set_invalid(self):
        for funit_type in self.values.copy():
            self.publish_state(funit_type, None)


class _SubgineriesFunit(Funit):
    entity_type = Subginery
    target_command = None
    target_class = None

    def set(self):
        for e in self.entity.engineries:
            if isinstance(e, self.target_class):
                try:
                    e.publish_provider_command(self.target_command, self.set_value, self.params, sig=self.signature)
                except CommandError as ex:
                    logging.warning(ex)
