import re
import time
from spread_core.bam.validation.base import corr_req_par


class HeaderFirmwareValidator:
    def __init__(self, data):
        if not isinstance(data, dict):
            raise ValueError('Wrong data structure (dict needed)')

        if not corr_req_par('type', str, data) or data['type'] not in ('Commercial', 'Demo'):
            raise ValueError('Wrong firmware type')

        # key must be GUID
        if not corr_req_par('key', str, data) \
                or not re.match(r'[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}$', data['key']):
            raise ValueError('Wrong firmware key')

        if not corr_req_par('date', str, data):
            raise ValueError('Wrong firmware date')
        try:
            time.strptime(data['date'], '%Y-%m-%dT%H:%M:%S.%f')
        except:
            raise ValueError('Wrong firmware date')

        if not corr_req_par('build', int, data) or data['build'] < 1:
            raise ValueError('Wrong firmware build')

        if not corr_req_par('builder', str, data):
            raise ValueError('Wrong firmware builder')

        if not corr_req_par('user', str, data):
            raise ValueError('Wrong firmware user')
