# import json
import re
from spread_core.bam.validation.base import corr_par, corr_req_par, ip
import spread_core.bam.validation.constraints as constr

class ManagerValidator:
    def __init__(self, data):
        if not isinstance(data, dict):
            raise ValueError('Wrong data structure (dict needed)')
        if not corr_req_par('type', str, data) or data['type'] not in constr.MAN_PROV_CONSTR:
            raise ValueError('Wrong type')
        if not corr_req_par('id', int, data) or data['id'] < 1:
            raise ValueError('Wrong id')
        # key must be GUID
        if not corr_req_par('key', str, data) \
                or not re.match(r'[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}$', data['key']):
            raise ValueError('Wrong key')
        if not corr_req_par('serverID', int, data) or data['serverID'] < 1:
            raise ValueError('Wrong serverID')
        if not corr_par('name', str, data):
            raise ValueError('Wrong name')
        self.Attributes(data)

    def Attributes(self, data):
        error = f"Wrong {data['type']} attribute"
        if data['type'] == 'Teletask':
            if 'attributes' not in data:
                raise ValueError(f"{error}s")
            if 'ip' not in data['attributes'] or not ip(data['attributes']['ip']):
                raise ValueError(f"{error} ip")
            if not corr_req_par('port', int, data['attributes']) \
                    or data['attributes']['port'] < 0 or data['attributes']['port'] > 65535:
                raise ValueError(f"{error} port")

        elif data['type'] in ('Dali', 'Mercury', 'Knx'):
            if 'attributes' not in data:
                raise ValueError(f"{error}s")
            if not corr_req_par('bus', str, data['attributes']):
                raise ValueError(f"{error} bus")
            if not corr_req_par('pollRate', int, data['attributes']) or data['attributes']['pollRate'] < 0:
                raise ValueError(f"{error} bus")

        elif data['type'] == 'Lom':
            if 'attributes' not in data:
                raise ValueError(f"{error}s")
            if not corr_req_par('host', str, data['attributes']):
                raise ValueError(f"{error} host")
            if not corr_req_par('port', int, data['attributes']) \
                    or data['attributes']['port'] < 0 or data['attributes']['port'] > 65535:
                raise ValueError(f"{error} port")
            if not corr_req_par('ssl', bool, data['attributes']):
                raise ValueError(f"{error} ssl")
            if not corr_req_par('login', str, data['attributes']):
                raise ValueError(f"{error} login")
            if not corr_req_par('password', str, data['attributes']):
                raise ValueError(f"{error} password")
            if not corr_req_par('upLink', str, data['attributes']):
                raise ValueError(f"{error} uplink")
            if not corr_req_par('downLink', str, data['attributes']):
                raise ValueError(f"{error} downlink")
            if not corr_req_par('latitude', float, data['attributes']):
                raise ValueError(f"{error} latitude")
            if not corr_req_par('longitude', float, data['attributes']):
                raise ValueError(f"{error} longitude")
            if not corr_req_par('altitude', int, data['attributes']):
                raise ValueError(f"{error} altitude")
            if not corr_req_par('broadcast', str, data['attributes']) \
                    or not re.match(r'[0-9A-F]+$', data['attributes']['broadcast']):
                raise ValueError(f"{error} broadcast")

        elif data['type'] == 'Ews':
            if 'attributes' not in data:
                raise ValueError(f"{error}s")
            if not corr_req_par('domain', str, data['attributes']):
                raise ValueError(f"{error} domain")
            if not corr_req_par('url', str, data['attributes']):
                raise ValueError(f"{error} url")
            if not corr_req_par('login', str, data['attributes']):
                raise ValueError(f"{error} login")
            if not corr_req_par('password', str, data['attributes']):
                raise ValueError(f"{error} password")
            if not corr_req_par('timeZoneIndex', int, data['attributes']):
                raise ValueError(f"{error} timeZoneIndex")
            if not corr_req_par('distributionGroup', str, data['attributes']):
                raise ValueError(f"{error} distributionGroup")
            if not corr_req_par('pollRate', int, data['attributes']) or data['attributes']['pollRate'] < 0:
                raise ValueError(f"{error} bus")
            if not corr_req_par('daysToPasswordExpirationWarning', int, data['attributes']) \
                    or data['attributes']['daysToPasswordExpirationWarning'] < 0 \
                    or data['attributes']['daysToPasswordExpirationWarning'] > 255:
                raise ValueError(f"{error} daysToPasswordExpirationWarning")

        elif 'attributes' in data:
            raise ValueError(f"{error}s")
