import logging

import spread_core.mqtt as mqtt
from spread_core.bam.equipment import AlarmEquipment
from spread_core.errors.project_errors import EngineryError
from spread_core.tools.service_launcher import Launcher


class AlarmingEquipmentLauncher(Launcher):

    @property
    def manager(self):
        if self._manager is None:
            self._manager = AlarmEquipment(self.project_id, self.mqttc)
        return self._manager

    def on_message(self, mosq, obj, msg):
        super(AlarmingEquipmentLauncher, self).on_message(mosq, obj, msg)
        if not msg.payload:
            return

        topic = mqtt.topic_of(msg.topic)
        try:
            payload = msg.payload.decode()
            if isinstance(topic, mqtt.spread.topic.SpreadTopic):
                var = mqtt.spread.variable.Variable.of(payload)
                if isinstance(topic, (mqtt.spread.topic.Set, mqtt.spread.topic.Renew)):
                    self._manager.on_command(topic, var)
                elif isinstance(topic, mqtt.spread.topic.State):
                    self._manager.on_state(topic, var)
            elif isinstance(topic, mqtt.project.ProjectFile):
                self._manager.on_project(topic, payload)
            else:
                logging.debug('[{}] => {}'.format(msg.topic, payload))
        except EngineryError as ex:
            logging.warning(ex)
        except BaseException as ex:
            logging.exception(ex)


def run():
    AlarmingEquipmentLauncher().connect()


if __name__ == '__main__':
    run()
