from . import bus, const, jocket, tros3, spread, project, dashboard
from .base import Topic, TopicError
from .classifier import LINEAR_CURVE, LOGARITHMIC_CURVE, classifier


""" 
Реализация через регулярные выражения работает на 58% долше:
Обработка 250`000 топиков занимает время:
методом split: 28.6 сек
методом regex: 49.2 сек
"""

"""
regex = re.compile(
    r'^'
    r'(Spread)/'
    r'  (Set|State|Renew)/'  # action
    r'  (\d+)/'  # project_id
    r'  (Hardware|Location|Equipment)/'  # e_family
    r'  ([\w\d]+)/(\d+)/'  # e_type, e_id
    r'  (?:([\w\d]+)/(\d+)/)?'  # e_type, e_id
    r'  ([\w\d]+)$'  # funit

    r'|(Bus)/'
    r'  (?:'
    r'    (Send|Dump)/(.+)$'  # direction, subtopic
    r'  |'
    r'    (State)/(.+)/([\w\d]+)$'  # State-direction, subtopic, funit
    r'  )'

    r'|(Project)/'
    r'  (\d+)/'  # project_id, 
    r'  (?:'
    r'    (File)/(.+)$'  # project_family, file_name
    r'  |'
    r'    (Ctp)$'  # project_family
    r'  |'
    r'    (Hash)$'  # project_family
    r'  |'
    r'    (Local)/(.+)$'  # project_family
    r'  )',
    re.VERBOSE
)


def topic_of(topic):
    match = regex.match(topic)
    if match:
        return [_ for _ in match.groups() if _]

    raise ValueError("Topic is unmatched: %s" % topic)
"""


def topic_of(topic):
    protocol, *parts = topic.split('/')

    try:
        if bus.TopicBus.protocol == protocol:
            return bus.TopicBus.decompose(*parts)

        elif spread.topic.SpreadTopic.protocol == protocol:
            return spread.topic.SpreadTopic.decompose(*parts)

        elif project.ProjectTopic.protocol == protocol:
            return project.ProjectTopic.decompose(*parts)

        elif protocol == tros3.topic.TROS3Topic.protocol:
            return tros3.topic.TROS3Topic.decompose(*parts)

        elif protocol == jocket.topic.JocketTopic.protocol:
            return jocket.topic.JocketTopic.decompose(*parts)

    except (TypeError, IndexError, ValueError):
        raise TopicError('Invalid topic: "%s"' % topic)

    raise TopicError('Unknown protocol: "%s"' % topic)
