from spread_core.mqtt.base import Topic, TopicError

BUS = "Bus"

STATE = "State"
DUMP = "Dump"
SEND = "Send"

STATUS = "Status"
INFO = "Info"


class TopicBus(Topic):
    protocol = BUS
    is_dump = False
    is_send = False
    is_state = False
    is_status = False
    is_info = False

    def __init__(self, direction, subtopic, funit=''):
        super(TopicBus, self).__init__()

        if not subtopic:
            raise TopicError('Empty subtopic')

        self.direction = direction
        self.part = subtopic
        self.funit = funit

        if direction == DUMP:
            self.is_dump = True
        elif direction == SEND:
            self.is_send = True
        elif direction == STATE:
            self.is_state = True
            self.funit = funit
            if funit == STATUS:
                self.is_status = True
            elif funit == INFO:
                self.is_info = True
            else:
                raise TopicError('Unknown FUnit: "%s"' % funit)

        else:
            raise TopicError('Unknown direction: "%s"' % direction)

    def __str__(self):
        s = f'{self.protocol}/{self.direction}/{self.part}'

        if self.funit:
            s += f'/{self.funit}'

        return s

    @classmethod
    def decompose(cls, direction: str, *subtopic: str):
        funit = ''

        if direction == STATE:
            funit = subtopic[-1]
            subtopic = subtopic[:-1]

        return cls(direction, '/'.join(subtopic), funit=funit)

    @classmethod
    def of_send(cls, part): return cls(SEND, part)

    @classmethod
    def of_dump(cls, part): return cls(DUMP, part)

    @classmethod
    def of_status(cls, part): return cls(STATE, part, STATUS)

    @classmethod
    def of_info(cls, part): return cls(STATE, part, INFO)
