import sys
import logging
import asyncio
from asyncio_mqtt import Client, MqttError

host = 'localhost'
project_id = None
LA_set = {}
# LA_set = {can: {module: {channel: {group: LA_id}}}}


async def main():
    while True:
        logging.info(f'Service started (host={host}, project_id={project_id})')
        try:
            async with Client(
                    hostname=host,
                    port=1883,
                    username='admin',
                    password='broker',
                    client_id='btn4la'
            ) as client:
                async with client.messages() as messages:
                    for can in LA_set:
                        for module in LA_set[can]:
                            for channel in LA_set[can][module]:
                                await client.subscribe(f'Bus/Dump/Rapida/{can}/Dali/{module}/{channel}')
                                logging.info(f'Subscribe to "Bus/Dump/Rapida/{can}/Dali/{module}/{channel}"')

                    async for message in messages:
                        try:
                            data = message.payload.decode().split('#')[0]

                            if len(data) == 4:
                                data = bytes.fromhex(data)
                            else:
                                continue
                        except:
                            continue

                        sb = data[0] & 0b00000001
                        if sb == 0 and (data[1] in (0, 254)):  # DAPC(0) or DAPC(254) command
                            addr = data[0] >> 1
                            if 64 <= addr <= 79:  # Group address
                                group = addr & 0b00001111
                                topic = message.topic.value.split('/')
                                try:
                                    can = int(topic[3])
                                    module = int(topic[5])
                                    channel = int(topic[6])
                                    LA_id = LA_set[can][module][channel][group]
                                except:
                                    continue

                                if data[1] == 0:
                                    payload = '{"value": false}'
                                    action = 'Off'
                                else:
                                    payload = '{"value": true}'
                                    action = 'On'

                                await client.publish(f'Spread/Set/{project_id}/Equipment/LightingArea/{LA_id}/On', payload, retain=False)
                                logging.info(f'can{can}: DAPC({data[1]}) command to Group({group}) -> LA({LA_id}) {action}')

        except MqttError as error:
            logging.info(error)
            await asyncio.sleep(3)


def info():
    print('Скрипт для управления LA через групповые команды DAPC(0) и DAPC(254)')
    print('Пример для запуска: python3 btn4la.py settings.txt')


def settings_info():
    print('Формат файла с настройками:\n'
          '  project_id=<id>\n'
          '  host=<address> (по умолчанию host=localhost)\n'
          '  <can> <module> <channel> <group> <LA_id>')


if __name__ == '__main__':
    logging.basicConfig(format=u'%(levelname)-8s [%(asctime)s]  %(message)s', level=logging.INFO, stream=sys.stdout)

    if len(sys.argv) == 1:
        info()
        print('Необходимо указать файл с настройками в качестве параметра.')
        settings_info()
        sys.exit(100)

    if sys.argv[1] in ('help', '/h', '/?'):
        info()
        settings_info()
        sys.exit(100)

    try:
        i = 0
        with open(f'{sys.argv[1]}', 'r') as file:
            for s in file:
                try:
                    i += 1
                    s = s.split()
                    if len(s) == 0 or s[0].startswith('#'):
                        continue

                    if s[0].startswith('project_id='):
                        project_id = int(s[0].replace('project_id=', ''))
                        continue

                    if s[0].startswith('host='):
                        host = s[0].replace('host=', '')
                        continue

                    can, module, channel, group, LA_id = [int(n) for n in s]
                except:
                    print(f'Ошибка в файле настроек (строка {i})')
                    sys.exit(100)

                if can not in LA_set:
                    LA_set[can] = {}

                if module not in LA_set[can]:
                    LA_set[can][module] = {}

                if channel not in LA_set[can][module]:
                    LA_set[can][module][channel] = {}

                LA_set[can][module][channel][group] = LA_id
    except:
        print('Файл с настройками не указан или указан неверно.')
        sys.exit(100)

    msg = []
    if project_id is None:
        msg.append('project_id')

    if not LA_set:
        msg.append('настройки для управления LA')

    if msg:
        print('Файл настроек не содержит ' + ', '.join(msg) + '.')
        settings_info()
        sys.exit(100)

    try:
        asyncio.run(main())
    except KeyboardInterrupt:
        logging.info('Service breaked')
        sys.exit(100)
