from unittest import TestCase

from spread_core.mqtt.spread.address import ProviderAddress

from spread_core.bam import engineries

from spread_core.bam.recipe import Recipe
from spread_core.mqtt.spread.topic import State
from spread_core.mqtt.spread.variable import Variable


class TestRecipe(TestCase):
    def setUp(self) -> None:
        self.handled_states = []

        recipe = Recipe(1, "Manager", 2, "Provider", 3)
        for i in range(11):
            if i < 5:
                e = engineries.LightingArea(i)
            else:
                e = engineries.DimmingLight(i)

            e.on_state = lambda topic, variable: self.handled_states.append((topic, variable))
            recipe.set_ingredient(e)

        self.recipe = recipe

    def emulate_state(self, funit, value):
        self.recipe.dispatch_state(
            State(ProviderAddress(1, "Manager", 2, "Provider", 3, funit)),
            Variable(value)
        )

    def test_address(self):
        self.assertEqual('1/Hardware/Manager/2/Provider/3/SomeFunit', self.recipe.get_address("SomeFunit"))

    def test_bonding(self):
        def is_bonded():
            return self.recipe.is_bonded

        self.assertTrue(is_bonded())
        self.recipe._engineries = []
        self.assertFalse(is_bonded())

    def test_ingredient_storing(self):
        self.assertEqual(5, len(self.recipe._las))
        self.assertEqual(6, len(self.recipe._engineries))

    def _test_level_state_handling(self, funit, value):
        self.emulate_state(funit, value)
        self.assertEqual(value, self.recipe.level)
        self.assertEqual(11, len(self.handled_states))

    def _test_guard_state_handling(self, funit, value):
        self.emulate_state(funit, value)
        self.assertEqual(value, self.recipe.guarded)
        self.assertEqual(11, len(self.handled_states))

    def test_level_state_handling(self):
        self._test_level_state_handling('BrightnessLevel', 321)

    def test_lum_state_handling(self):
        self._test_level_state_handling('CurrentLuminosity', 123)

    def test_presence_state_handling(self):
        self._test_level_state_handling('CurrentPresence', "SomePresenceValue")

    def test_on_state_handling(self):
        self._test_guard_state_handling('On', "OnValue")
