from unittest import TestCase

from spread_core.bam.const import *
from spread_core.bam.generator import generate_enginery
from spread_core.errors.project_errors import ProjectError


class TestEngineryGeneration(TestCase):
    def _check_enginery_generation(self, provider_type, attributes=None, **attributes_for_check):
        enginery = generate_enginery(999, provider_type, {ATTRIBUTES: attributes or {}})

        self.assertEqual(enginery.id, 999)
        self.assertEqual(provider_type, type(enginery).__name__)

        for attr_name in attributes_for_check:
            try:
                self.assertEqual(getattr(enginery, attr_name), attributes_for_check[attr_name])
            except AttributeError as ex:
                self.fail(str(ex))

        return enginery

    def test_unknown_type(self):
        with self.assertRaises(ProjectError) as ex:
            self._check_enginery_generation('SomeNonEexistType', {})

        self.assertEqual(ex.exception.args[0], 'Unknown Enginery type "SomeNonEexistType(999)"')

    def test_invalid_power(self):
        with self.assertRaises(ProjectError) as ex:
            self._check_enginery_generation('DimmingLight', {'power': 'invalidInt'})

        self.assertEqual(ex.exception.args[0], "DimmingLight(999): invalid literal for int() with base 10: 'invalidInt'")

    def test_missing_power(self):
        self._check_enginery_generation('DimmingLight', power=0)

    def test_dimmer(self):
        self._check_enginery_generation("DimmingLight", {'power': 99}, power=99)

    def test_relay(self):
        self._check_enginery_generation("SwitchingLight", {'power': 99}, power=99)

    def test_rgb(self):
        self._check_enginery_generation("RgbLight", {'power': 99}, power=99)

    def test_dynamic(self):
        self._check_enginery_generation("DynamicLight", {'power': 99}, power=99)

    def test_la(self):
        self._check_enginery_generation("LightingArea")

    def test_presence_sensor(self):
        self._check_enginery_generation("PresenceSensor")

    def test_light_sensor(self):
        self._check_enginery_generation("LightSensor")

    def test_scenario(self):
        self._check_enginery_generation("Scenario")

    def test_shutter(self):
        self._check_enginery_generation("Shutter")

    def test_curtain(self):
        self._check_enginery_generation("Curtain")

    def test_blind(self):
        self._check_enginery_generation("Blind")
