from unittest import TestCase

from spread_core.bam.engineries import Enginery
from spread_core.bam.subgineries.subginery import Subginery
from spread_core.errors.project_errors import EntityNotPresent
from spread_core.tests.equipment.utils import TestEngineryRecipe
from spread_core.warehouse.equipment_warehouse import EquipmentWarehouse


class TestEquipmentWarehouse(TestCase):
    def setUp(self) -> None:
        self.store = EquipmentWarehouse(1)
        self.store.add_manager(2, "Manager", )
        self.store.add_provider(TestEngineryRecipe(3))
        self.store.add_subginery(Subginery(4, 99, []))
        self.store.add_enginery(Enginery(5), 3, "SomeIngredientType")

    def test_subginery_selecting(self):
        sb = Subginery(999, 99, [])
        self.store.add_subginery(sb)
        self.assertIs(sb, self.store.get_entity(99))

    def test_recipe_selecting(self):
        r = TestEngineryRecipe(999)
        self.store.add_provider(r)
        self.assertIs(r, self.store.get_recipe(2, 999))

    def test_manager_selecting(self):
        self.store.add_manager(999, "Manager")
        manager_data = self.store.get_manager(999)

        self.assertEqual(999, manager_data['id'])
        self.assertEqual('Manager', manager_data['type'])
        self.assertEqual({}, manager_data['providers'])

    def test_enginery_selecting(self):
        e = Enginery(999)
        self.store.add_enginery(e, 3, "SomeIngredientType")
        self.assertIs(e, self.store.get_entity(999))

    def test_non_existent(self):
        self.assertRaises(EntityNotPresent, self.store.get_entity, 1000)
        self.assertRaises(EntityNotPresent, self.store.get_manager, 1000)
        self.assertRaises(EntityNotPresent, self.store.get_recipe, 2, 1000)
