from spread_core.mqtt import project
from spread_core.tests.topics.test_common import TestTopic


class TestProjectTopic(TestTopic):
    def test_invalid_topic(self):
        self.parse_invalid_topic('Project', 'Invalid topic: "Project"')
        self.parse_invalid_topic('Project/0', 'Invalid topic: "Project/0"')
        self.parse_invalid_topic('Project/0/qwe', 'Unknown family: "qwe"')
        self.parse_invalid_topic('Project/0/File', 'Invalid topic: "Project/0/File"')
        self.parse_invalid_topic('Project/0/Local', 'Invalid topic: "Project/0/Local"')
        self.parse_invalid_topic('Project/qwe/Ctp', 'Invalid topic: "Project/qwe/Ctp"')


class TestCtpTopic(TestTopic):
    def test_string_casting(self):
        self.assertEqual(project.ProjectFile.ctp(0), 'Project/0/Ctp')

    def test_string_parsing(self):
        topic = self.PARSE('Project/0/Ctp')
        self.assertIsInstance(topic, project.ProjectFile)
        self.assertEqual(topic.project_id, 0)
        self.assertEqual(topic.file_name, 'Ctp')


class TestFileTopic(TestTopic):
    def test_string_casting(self):
        self.assertEqual(project.ProjectFile.locations(0), 'Project/0/File/locations.json')
        self.assertEqual(project.ProjectFile(0, 'some.file.qwe'), 'Project/0/File/some.file.qwe')

    def test_string_parsing(self):
        topic = self.PARSE('Project/0/File/some.file')
        self.assertIsInstance(topic, project.ProjectFile)
        self.assertEqual(topic.project_id, 0)
        self.assertEqual(topic.file_name, 'some.file')
