from collections import deque


class Popup(Exception):
    pass


class TestMqttClient:
    def __init__(self):
        self.topics = deque()
        self.messages = deque()

    def publish(self, topic, data, retain=False, delay=0, log_level=1, **kwargs):
        self.messages.append((topic, data, retain, delay, log_level, kwargs))

    def subscribe(self, topic, log=True):
        self.topics.append(topic)

    def unsubscribe(self, topic):
        try:
            self.topics.remove(topic)
        except ValueError:
            pass


class TestCanBus:
    def __init__(self, channel=None, *args, **kwargs):
        self.channel = channel

    def send(self, *args, **kwargs):
        pass

    def shutdown(self):
        pass

    def __iter__(self):
        yield from ()


class Handler(Exception):
    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs

    @classmethod
    def proceed(cls, *args, **kwargs):
        raise cls(*args, **kwargs)

    @staticmethod
    def check_handle_count(action, count, *args, **kwargs):
        gen = action(*args, **kwargs)
        current_count = 0
        for i in range(count):
            try:
                next(gen)
                current_count += 1
            except StopIteration:
                raise CountException(current_count, count)

        try:
            next(gen)
        except StopIteration:
            pass
        else:
            current_count += 1
            raise CountException(current_count, count)


class CountException(Exception):
    def __init__(self, expected_count, real_count):
        super(CountException, self).__init__(f'Expected count is {expected_count} but {real_count} has occurred')
        self.expected_count = expected_count
        self.real_count = real_count


class TestFunit(Exception):
    def __init__(self, entity, variable):
        self.entity = entity
        self.value = variable

    @classmethod
    def of(cls, _):
        return cls

    def set(self):
        super(TestFunit, self).__init__("Set action executed")
        raise self

    def get(self):
        super(TestFunit, self).__init__("Get action executed")
        raise self


class TestStore(object):
    """Имитация хранимлища провайдеров"""

    def get_provider(self, provider_id):
        raise Popup(provider_id)
