from unittest import TestCase

from spread_dali.address import Short, Group, InstanceType, InstanceNumber, InstanceGroup, InstanceBroadcast
from spread_dali.device.events import Event, frame_of_sources

generate_frame = frame_of_sources
parse_frame = Event.of_frame


class TestDaliEvents(TestCase):
    def test_event_frame_generator_of_negative_schema(self):
        for source1, source2 in ((Short(0), InstanceGroup(0)),
                                 (Short(0), InstanceBroadcast()),

                                 (Group(0), InstanceGroup(0)),
                                 (Group(0), InstanceNumber(0)),
                                 (Group(0), InstanceBroadcast()),

                                 (InstanceType(0), InstanceGroup(0)),
                                 (InstanceType(0), InstanceType(0)),
                                 (InstanceType(0), InstanceBroadcast()),

                                 (InstanceGroup(0), InstanceGroup(0)),
                                 (InstanceGroup(0), InstanceNumber(0)),
                                 (InstanceGroup(0), InstanceBroadcast()),

                                 (InstanceNumber(0), InstanceGroup(0)),
                                 (InstanceNumber(0), InstanceType(0)),
                                 (InstanceNumber(0), InstanceBroadcast())):

            with self.assertRaises(ValueError):
                generate_frame(source1, source2, 0)

    def test_event_frame_generator_of_negative_value(self):
        for value in -1, 1024, None, {}, [], object:
            with self.subTest(event_value=value):
                with self.assertRaises(ValueError):
                    generate_frame(Short(0), InstanceType(1), value)

    def test_event_frame_parser_of_negative_frame_length(self):
        for frame_length in range(5):
            if frame_length == 3:
                continue
            with self.subTest(frame_length=frame_length):
                with self.assertRaises(ValueError):
                    parse_frame(bytes(frame_length))

    def test_event_frame_parser_of_negative_frame_type(self):
        for frame in ('123', [1, 2, 3], None, True, 0xFFFFFF, -1):
            with self.subTest(frame_type=type(frame)):
                with self.assertRaises(ValueError):
                    parse_frame(frame)

    def test_event_frame_parser_of_negative_flags(self):
        for frame in (0b000000010000000000000000,
                      0b110000001000000000000000):
            with self.subTest(frame=frame):
                with self.assertRaises(ValueError):
                    parse_frame(frame.to_bytes(3, 'big'))

    def test_event_frame_parser(self):
        for source1, source2 in ((Short(63), InstanceType(31)),
                                 (Short(63), InstanceNumber(31)),
                                 (Group(31), InstanceType(31)),
                                 (InstanceType(31), InstanceNumber(31)),
                                 (InstanceGroup(31), InstanceType(31))):

            event_frame = generate_frame(source1, source2, 1023)
            event = parse_frame(event_frame)

            self.assertEqual(event.source1, source1)
            self.assertEqual(event.source2, source2)
            self.assertEqual(event.value, 1023)
