from datetime import date

from django.shortcuts import render
from django.utils.translation import gettext_lazy as _

from spread_dashboard.project import models
from spread_dashboard.project.bam import broker_auth
from spread_dashboard.commissioning.views import make_area_context, make_location_context


def for_print(request):
    # index_locations = tree_with_areas(location_id, 2)  # дерево для отображения содержания
    project = models.Header.objects.first()
    location_id = request.GET.get('location')
    location_id = int(location_id) if location_id else None
    flat_locations = list(flat_list_with_areas(project, location_id, 2))  # плоский список для рендеринга страниц

    available_navigation = set()
    if location_id:
        for item in flat_locations:
            id_ = item['area']['id'] if 'area' in item else item['location']['id']
            available_navigation.add(id_)

    cover = flat_locations and flat_locations[0]['location'].get('preview')  # картинку корневой локации на обложку
    from_date = date.fromisoformat(request.GET.get('dateFrom'))
    to_date = date.fromisoformat(request.GET.get('dateTo'))
    project_name = project.name
    location_name = location_id and flat_locations[0]['location'].get('name')
    # закинем на уровень с корневой локацией введение и глоссарий
    flat_locations.insert(0, {'intro_text': {'id': 'glossary', 'name': _('glossary'), 'counter': 2, 'level': 0}})
    flat_locations.insert(0, {'intro_text': {'id': 'intro', 'name': _('intro'), 'counter': 1, 'level': 0}})

    return render(request, 'commissioning/printer/index.html', context={
        # 'slices': (f'{i}:{i+column_size}' for i in range(0, len(flat_locations), column_size)),  # для облегчения посторения оглавления в шаблонизаторе
        'flat_locations': flat_locations,
        'project_name': project_name,
        'location_name': location_name,
        'broker': broker_auth(),
        'cover': cover,
        'from_date': from_date.strftime('%d.%m.%Y'),
        'to_date': to_date.strftime('%d.%m.%Y'),
        'dateFrom': request.GET.get('dateFrom'),
        'dateTo': request.GET.get('dateTo'),
        'available_navigation': available_navigation,
    })


def tree_with_areas(location_id: int = None, counter=0) -> list:

    def make_item(location: models.Location) -> dict:
        nonlocal counter
        counter += 1
        local_counter = counter
        areas = []
        for area in location.engineries(('LightingArea',)):
            counter += 1
            areas.append({'id': area.id, 'name': area.name, 'type': 'area', 'counter': counter})
        return {
            'id': location.id,
            'name': location.name,
            'type': 'location',
            'areas': areas,
            'locations': [make_item(sub) for sub in location.sub_locations],
            'counter': local_counter
        }

    loc = location_by_id(location_id)
    return [make_item(loc)]


def flat_list_with_areas(project, location_id: int = None, counter=0) -> list:
    def locations_and_areas(location, level=0) -> list:
        nonlocal counter
        counter += 1
        # local_counter = counter
        result_list = [make_location_context(location.id, project.project_id, counter, level)]
        areas = []
        area_level = level + 1
        for area in location.engineries(('LightingArea',)):
            counter += 1
            areas.append(make_area_context(location.id, area.id, project.project_id, counter, area_level))
        result_list += areas
        level += 1
        for loc in location.sub_locations:
            result_list += locations_and_areas(loc, level)
        yield from result_list

    loc = location_by_id(project, location_id)
    yield from locations_and_areas(loc)


def location_by_id(project, location_id: int = None) -> models.Location:
    if location_id:
        try:
            return models.Location.objects.get(id=location_id)
        except models.Location.DoesNotExist:
            pass

    return models.Location.objects.get(id=project.root_location)
