/* Базовый объект приложения Vue.js
содержит в себе объект вебсокета.
провайдит методы для подписки / отписки и публикации
 */

let BasicApp = {
    delimiters: ['{|', '|}'],
    data() {
        return {
            connected: false,
            subTopics: [],
            URL: '',
        }
    },
    provide() {
        return {
            subscribe: this.subscribe,
            unsubscribe: this.unsubscribe,
            publishMessage: this.publishMessage,
        }
    },
    methods: {
        onMessage(event) {
            let payload = JSON.parse(event.data);
            for (let key in payload) {
                if (key == 'error') {
                  console.error(`${key}: ${JSON.stringify(payload[key])}`);
                  return;
                }
                this.processStateMessage(key, payload);
                this.processReplyMessage(key, payload);
            }
        },
        subscribe(subs) {
            if (Array.isArray(subs)) {
                subs = {subscriptions: Array.from(subs)};
            }
          this.webSocket.send(JSON.stringify(subs));
        },
        unsubscribe(unsubs) {
            if (Array.isArray(unsubs)) {
                unsubs = {unsubscriptions: unsubs};
            }
          this.webSocket.send(JSON.stringify(unsubs));
        },
        publishMessage(pubs) {
            if (Array.isArray(pubs)) {
                pubs = {publications: pubs};
            }
          this.webSocket.send(JSON.stringify(pubs));
        },
        updateSubTopics(data) {
            this.subTopics.push(...data);
        },
        processStateMessage(key, payload) {
            if (!key.includes('State')) {
                return;
            }
            // если ключом является полное имя топика, то заберем только имя параметра
            let field = key.split('/').pop();

            if('invalid' in payload[key]) {  // невалидные значения не устанавливаем
              console.error(`${field}: Невалидное значение. Проверьте работоспособность объекта`);
              return;
            }
            console.warn(`STATE ${field}: ${payload[key].value}`);
            this[field] = payload[key].value;
        },
        processReplyMessage(key, payload) {
            let field = payload[key].key;
            if(!key.includes('Reply')) {
                return;
            }
            if('invalid' in payload[key]) {  // невалидные значения не устанавливаем
              console.error(`${field}: Невалидное значение. Проверьте работоспособность объекта`);
              return;
            }
            this[field] = payload[key]
            console.warn(`REPLY ${field}: ${payload[key]}`);
        },
        httpGetRequest(url, async=false) {
            // GET-запрос на указанный url. Параметры необходимо передавать также в url
            let resp = {
                result: null,
                error: null
            }
            let request = new XMLHttpRequest();
            request.open('GET',url, async);
            request.withCredentials = true
            request.send('');
            if (request.status == 200) {
                resp.result = JSON.parse(request.responseText);
            } else {
                resp.error = request.responseText;
                console.error(resp.error);
            }
            return resp;
        }
    },
    created() {
        this.webSocket = new WebSocket(this.URL);
        this.webSocket.onopen = () => {
          // console.log(`Сonnected to ${this.URL}`);
          this.connected = true;
          this.subscribe(this.subTopics);
        }
        this.webSocket.onmessage = this.onMessage;
        this.webSocket.onclose = () => {
            this.connected = false;
        };
    },
}
