// Основное приложение, управляющее текстовыми полями

let App = {
    extends: BasicApp,
    el: '#meter-app',
    data: {
        // Поля, уникальные для страницы
        RateAmperage: undefined,
        RateVoltage: undefined,
        SerialDate: {},
        Voltage: {}, // напряжение по фазам
        Amperage: {}, // ток по фазам
        Angle: {}, // углы между фазами
        Frequency: undefined,
        LoadOn: true, // управление нагрузкой (рубануть электричество)
        PowerActive: {},
        PowerReactive: {},
        PowerTotal: {},
        PowerRate: {},
        Validity: true,
        config: {},
        StoredEnergy: {},
        StoredEnergyStart: {},
        StoredEnergyEnd: {},
        periods: {},
        password: undefined,  // пароль от управления нагрузкой
        passwordModal: undefined, // управление модальным окном проверки пароля для выкл / вкл нагрузки
    },
    methods: {
        stopSwitch(event) {
            // при попытке переключении чек-бокса стопаем анимацию
            // его состояние берем из переменной LoadOn
            event.preventDefault();
        },
        setLoad() {
            let vm = this;
            let password = vm.password;
            vm.password = '';  // очищаем поле
            axios(`/spread/mercury/check_password?id=${vm.config.entity_id}&password=${password}&loadon=${vm.LoadOn}`)
                .then(function (response) {
                    if ('error' in response.data) {
                        let pass = vm.$refs.password;
                        pass.setCustomValidity(response.data.error);
                        pass.reportValidity();
                    } else {
                        vm.passwordModal.hide();
                    }
                });

        },
    },
    created() {
        let expr = /(?:id=)(\d+)/;
        let meterId = window.location.search;
        meterId = meterId.match(expr);
        meterId = meterId && meterId[meterId.length-1];
        // тут проще получить синхронно
        let request = new XMLHttpRequest();
        request.open('GET','/spread/mercury/config_api/?meter=' + meterId, false);
        request.withCredentials = true
        request.send('');
        if (request.status == 200) {
            let resp = JSON.parse(request.responseText);
            this.config = resp.config;
            this.periods = resp.periods;
        } else {
            console.error(request.responseText)
        }
    },
    mounted() {
        this.passwordModal = new bootstrap.Modal('#password-meter', {});
    },
    watch: {
        config: function (value) {
            let reply = this.config.reply;
            this.subTopics = [
                {topic: this.config.state},
                {topic: reply}
            ]
        },
        connected: function (state) {
            if (state) {
                this.subscribe(this.subTopics);
            }
        },
    }
};
