/* Панель с энергопотреблением по тарифам */

const datePickerParams = {
    opens: 'center',
    endDate: moment().subtract(1, 'days'), // счетчик НЕ возвращает нарастающий итог с текущими сутками
    startDate: moment().subtract(1, 'days').startOf('month'), // начало месяца, возможного прошлого
    //autoApply: true,
    maxDate: moment().subtract(1, 'days'),
    minDate: moment().subtract(123, 'days'),
    locale: {
        maxDate: moment(),
        format: 'DD.MM.YYYY',
        monthNames: ['Январь', 'Февраль', 'Март', 'Апрель', 'Май', 'Июнь', 'Июль', 'Август', 'Сентябрь', 'Октябрь', 'Ноябрь', 'Декабрь'],
        separator: " - ",
        applyLabel: "Применить",
        cancelLabel: "Отменить",
        fromLabel: "From",
        toLabel: "To",
        customRangeLabel: "Custom",
        weekLabel: "Н",
        daysOfWeek: ["Вс", "Пн", "Вт", "Ср", "Чт", "Пт", "Сб"],
        firstDay: 1,
    }
};

let EnergyPanel = {
    delimiters: ['{|', '|}'],
    props: {
        StoredEnergy: Object, // готовые периоды энергопотребление
        StoredEnergyEnd: Object, // данные конца кастомного периода
        StoredEnergyStart: Object, // данные начала кастомного периода
        connected: Boolean,
        periods: Object,
    },
    data: function() {
        return {
            periodObj: {period: 0, month: 0},
            chooseMessage: undefined,
            periodType: 'total',
            dateRange: {
                start: datePickerParams.startDate.format('YYYY-MM-DD'),
                end: datePickerParams.endDate.add(1, 'days').format('YYYY-MM-DD') // счетчик возвращает энергию на начало суток
            },
            showPicker: false, // флаг для скрытия элемента выбора дат, т.к. его нельзя удалять из DOM-дерева
            messages: {
                'total': undefined,
                'year': 'Выберите год',
                'month': 'Выберите месяц',
                'day': 'Выберите день',
                'custom': 'Выберите период'
            },
            totalEnergies: {
                active: null,
                active_reverse: null,
                reactive: null,
                reactive_reverse: null,
            },
            customEnergies: {
                active: null,
                active_reverse: null,
                reactive: null,
                reactive_reverse: null,
            },
        }
    },
    inject: ['publishMessage'],
    methods: {
        updatePanel(periodType) {
            this.periodType = periodType;
            this.chooseMessage = this.messages[periodType];
            if (periodType in this.periods) {
                this.showPicker = false;
                this.periodObj = this.periods[periodType][0];
                this.getPeriodEnergy(this.periodObj);
            } else {
                this.showPicker = true;
                this.getCustomPeriodEnergy()
            }

        },
        getPeriodEnergy(periodObj) {
            let root = this.$root;
            let payload = {
                params: {period: periodObj.period, month: periodObj.month, tariff: null},
                session: root.config.session_id,
                key: periodObj.key || 'StoredEnergy'
            }
            let pubs = [];
            let topic = root.config.get;
            topic = topic.replace('{}', 'StoredEnergy');
            for (let tariff=0; tariff<=4; tariff++) {
                payload.params.tariff = tariff;
                // в payload кладем копию объекта
                let payloadCopy = JSON.parse(JSON.stringify(payload));  // любите ли вы js, как "люблю" его я?
                pubs.push({topic: topic, payload: payloadCopy});
            }
            this.publishMessage(pubs);
        },
        getCustomPeriodEnergy() {
            /* вычислить потребление за кастомный период можно только
            вычтя из энергии конца периода энергию начала периода
            т.е. сделать запросы по 2-м датам и взять разницу
             */
            let vm = this;
            if (!vm.showPicker) {
                return;
            }
            let payload = {
                params: {},
                key: '',
                session: vm.$root.config.session_id,
            }
            let topic = vm.$root.config.get;
            topic = topic.replace('{}', 'StoredEnergy');
            let pubs = [];
            /* формируем массив запросов, чередуя конец периода по тарифу с началом, для всех тарифов.
            тогда в вотчере для StoredEnergyStart можно вычислить готовый объект для каждого тарифа
             */
            for (let tariff=0; tariff<=4; tariff++) {
                payload.params.period = vm.dateRange.end;
                payload.params.tariff = tariff;
                payload.key = 'StoredEnergyEnd';
                pubs.push({topic: topic, payload: JSON.parse(JSON.stringify(payload))});

                payload.params.period = vm.dateRange.start;
                payload.key = 'StoredEnergyStart';
                pubs.push({topic: topic, payload: JSON.parse(JSON.stringify(payload))});
            }
            this.publishMessage(pubs);
        },
        percent(part, total) {
            let x = (part/total)*100 || 0;
            x = Math.round(x).toString() + '%';
            return x;
        }
    },
    watch: {
        StoredEnergy: function(obj) {
            console.log(obj.params.tariff + ': ' + obj.value.active);
            let tariff = obj.params.tariff;
            let values = obj.value;
            for (let part in values) {
                let ref = part + '-' + tariff;
                let value = values[part];
                if (tariff == 0) {
                    this.totalEnergies[part] = value;
                }
                if (this.$refs[ref]) {
                    this.$refs[ref].textContent = value;
                }
                if (this.$refs[ref + '-progress']) {
                    let total = this.totalEnergies[part];
                    this.$refs[ref + '-progress'].style.width = this.percent(value, total);
                }
            }
        },
        StoredEnergyStart: function(obj) {
            // запросы выстроены так, что когда приходит StoredEnergyStart
            // можно вычесть его из StoredEnergyEnd и отдать в результат в StoredEnergy
            let end = this.StoredEnergyEnd.value;
            let start = obj.value;
            let computedObj = {params: {}, value: {}};
            computedObj.params.tariff = obj.params.tariff
            for (let part in start) {
                computedObj.value[part] = (end[part] - start[part]).toFixed(3);
            }
            this.StoredEnergy = computedObj;
        },
        connected: function(state) {
            if (state) {
                // по умолчанию рендерим вкладку Всего
                this.getPeriodEnergy(this.periodObj);
            }
        },
        periodObj: function(obj) {
            this.getPeriodEnergy(obj);
        },
        dateRange: function(v) {
            this.getCustomPeriodEnergy();
        }
    },
    mounted() {
        let format = 'YYYY-MM-DD';
        let vm = this;
        vm.picker = new DateRangePicker(
            'datepicker',
            datePickerParams,
            function (startDate, endDate) {
                vm.dateRange = {
                    start: startDate.format(format),
                    end: endDate.add(1, 'days').format(format) // счетчик возвращает энергию на начало суток
                };
            },
        );
    }
}
