class Address:
    __slots__ = ['control', 'state']

    def __init__(self, state, control):
        self.state = state
        self.control = control

    def binding_items(self):
        yield self.state
        yield self.control

    @classmethod
    def deserialize(cls, data):
        return cls(**data)

    def __str__(self):
        return f'<{self.state} {self.control}>'

    def __repr__(self):
        return self.__str__()

    def __hash__(self):
        return hash(self.state + self.control)

    def __eq__(self, other):
        if isinstance(other, Address):
            return self.state == other.state and self.control == other.control

        return False


class LightAddress(Address):
    pass


class MotorAddress(Address):
    __slots__ = Address.__slots__ + ['stop']

    def __init__(self, stop=None, **kwargs):
        super(MotorAddress, self).__init__(**kwargs)
        self.stop = stop

    def binding_items(self):
        yield from super(MotorAddress, self).binding_items()
        yield self.stop


class SensorAddress:
    __slots__ = ['state']

    def __init__(self, state):
        self.state = state

    def binding_items(self):
        yield self.state

    @classmethod
    def deserialize(cls, data):
        return cls(**data)

    def __str__(self):
        return f'<{self.state}>'

    def __repr__(self):
        return self.__str__()

    def __hash__(self):
        return hash(self.state)

    def __eq__(self, other):
        if isinstance(other, SensorAddress):
            return self.state == other.state

        return False
