import logging
import threading

from spread_core.tools.base_interface import AbstractInterfaceClass
from . import commands


class KnxInterface(AbstractInterfaceClass):
    wait_for_ready = True

    def __init__(self, *args, **kwargs):
        super(KnxInterface, self).__init__(*args, **kwargs)
        self._request_event = threading.Event()
        self.response = None
        self.response_address = None

    def send(self, command: commands.KnxFrame, state_address: str = None) -> commands.DPTU:
        while not self._stopped and not self.ready:
            threading.Event().wait(1)

        if self._stopped:
            return None

        logging.debug('-> %s [%s]' % (command, command.frame.hex().upper()))
        self.response_address = state_address
        self.response = command.dptu.empty()

        self._mqtt_client.publish(self._send_topic, command.frame_str)

        if state_address:
            self._request_event.wait(self.timeout)
            self._request_event.clear()
            yield self.response

        self.response = None

    def set_stopped(self):
        super(KnxInterface, self).set_stopped()

        if self._request_event and not self._request_event.is_set():
            self._request_event.set()

    def on_dump(self, response, *args, **kwargs):
        data, *flags = response.decode().split('#')
        frame = commands.KnxFrame.of(data)

        logging.debug('  <- %s [%s]' % (frame, data.upper()))

        if self.response and frame.address == self.response_address:
            self.response.fill(frame.dptu.frame)
            self._request_event.set()
        else:
            self.event_handler.send(frame)
